<?php

/**
 * The History abstract class
 *
 * This abstract class must be extended for the DeltaV Versioning Feature
 * It's based on the basic DAV filesystem, but IProperties support is *required*
 * With this you can make your own History filesytem as you want
 *
 * 
 * @package Sabre
 * @subpackage DeltaV
 * @copyright Copyright (C) 2012 PD Programmeur. All rights reserved.
 * @author Pierre Dubois (http://www.pdprogrammeur.com/) 
 * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
 */
abstract Sabre_DeltaV_History extends Sabre_DAV_Collection implements Sabre_DAV_IProperties {
	
	protected $versionPrefix = ""; // you can set also "V"
	
	protected $historyPrefix = "";
	
	protected $historyFolder = "his"; // Par rapport au spec
	
	protected $versionFolder = "ver"; // Par rapport au spec
	
	/**
	 * Create a new Ressource into the History folder
	 *
	 * This are an alias for createFile()
	 *
	*/
	public function createVersion($data = null){
		
		// Generate the name of the ressource
		$name = $this->versionPrefix + ($this->calculateNbFile() + 1);
		
		
		// Create the ressource
		$this->createFile($name, $data);
	}
	
	/**
	 * Create a new Ressource into the History folder
	 *
	 * This are an alias for createDirectory()
	 *
	*/
	public function createHistory(){
		
		$this->calculateNbCollectionFile();
		
		// Generate the name of the ressource
		$name = $this->historyPrefix + ($this->calculateNbCollection() + 1);
		
		// Create the ressource
		$this->createDirectory($name);
	}
	
	
	protected function calculateNbFile(){
		
		$nbFile = 0;
		
		foreach($this->getChildren() as $child) {
			if($child instanceof 'Sabre_DeltaV_Version'){
				$nbFile ++;
			}
		}
		
		return $nbFile;
	}
	
	protected function calculateNbCollection(){
		
		$nbCollection = 0;
		
		foreach($this->getChildren() as $child) {
			if($child instanceof 'Sabre_DeltaV_History'){
				$nbCollection ++;
			}
		}
		
		return $nbCollection;
	}
	

     /**
     * Returns a child object, by its name.
     *
     * This method makes use of the getChildren method to grab all the child nodes, and compares the name. 
     * Generally its wise to override this, as this can usually be optimized
     * 
     * @param string $name
     * @throws Sabre_DAV_Exception_FileNotFound
     * @return Sabre_DAV_INode 
     */
    public function getChild($name) {

        foreach($this->getChildren() as $child) {

            if ($child->getName()==$name) return $child;

        }
        throw new Sabre_DAV_Exception_FileNotFound('File not found: ' . $name);

    }

    /**
     * Checks is a child-node exists.
     *
     * It is generally a good idea to try and override this. Usually it can be optimized.
     * 
     * @param string $name 
     * @return bool 
     */
    public function childExists($name) {

        try {

            $this->getChild($name);
            return true;

        } catch(Sabre_DAV_Exception_FileNotFound $e) {

            return false;

        }

    }

    /**
     * Creates a new file in the directory 
     * 
     * @param string $name Name of the file 
     * @param resource $data Initial payload, passed as a readable stream resource. 
     * @throws Sabre_DAV_Exception_Forbidden
     * @return void
     */
    public function createFile($name, $data = null) {

        throw new Sabre_DAV_Exception_Forbidden('Permission denied to create file (filename ' . $name . ')');

    }

    /**
     * Creates a new subdirectory 
     * 
     * @param string $name 
     * @throws Sabre_DAV_Exception_Forbidden
     * @return void
     */
    public function createDirectory($name) {

        throw new Sabre_DAV_Exception_Forbidden('Permission denied to create directory');

    }


}

