<?php


// This is the backend class, managed for a file system repository

class IHistory {

	protected $historyPrefix = "";
	protected $historyFolder = "his"; // Par rapport au spec
	protected $versionPrefix = ""; // you can set also "V"
	protected $versionFolder = "ver"; // Par rapport au spec
	
	protected $path;
	
	private $dataDir;
	private $dataCache;

	public __construct($path, $dataDir){
		// The $path are the location of the filesystem repository
		$this->path = $path;
		
		// Set the History Folder
		$hisPath = $this->path . '/' . $this->historyFolder;
		
		// 1. Check if the history location exist
		if (!file_exists($hisPath)){
			mkdir($hisPath); // Create the history folder
		}
		
		// 2. Setup the dataDir
		$this->dataDir = $dataDir;
		
	}

	// Create a new history
	public function createHistory($uri, $body, $props){
		
		if($this->isUnderVersioning($uri)){
			// Return the latest version
			return $this->getVersion($uri);
		}
		
		$hisPath = $this->path . '/' . $this->historyFolder;
	
		// 1. Get nbDirectory for the "history" location
		$hisVersion = (getNbHistory() + 1);
		$hisLocation = $hisPath . '/' . $hisVersion);
		mkdir($hisLocation);
		
		// 2. Create the version folder
		$verLocation = $hisLocation . '/' . $this->versionFolder;
		mkdir($verLocation);
		
		// 3. Create the first history
		$verFilePath = $verLocation . '/1'  
		file_put_contents($verFilePath, $body);
		
		// 4. Cast the new file version into Sabre_DAV_FSExt_File 
		$verFile = new Sabre_DAV_FSExt_File($verFilePath);
		
		// 5. Add the file property
		$verFile->updateProperties($props);
		
		// ~ 6. Lock the file ??
		
		// 7. Register this new history for the uri given
		$this->registerUriVersion($uri, $hisVersion);
		
		// 8. Return the file object
		return $verFile;
		
	}
	
	public function createVersion($uri, $body, $props){
		
		$hisPath = $this->path . '/' . $this->historyFolder;
		
		// 1. Get the History 
		$hisVersion = $this->getHistoryByUri($uri);
		$hisLocation = $hisPath . '/' . $hisVersion);
		
		// 2. Calculate the new filename
		$newVer = $this->getNbVersion($hisLocation) + 1;
		
		// 3. Create the versionned file
		$verLocation = $hisLocation . '/' . $this->versionFolder;
		$verFilePath = $verLocation . '/' . $newVer;
		if(file_exists($verFilePath)) throw new Sabre_DAV_Exception_Forbidden('The version file already exist');
		file_put_contents($verFilePath, $body);
		
		// 4. Cast the new file version into Sabre_DAV_FSExt_File 
		$verFile = new Sabre_DAV_FSExt_File($verFilePath);
		
		// 5. Add the file property
		$verFile->updateProperties($props);
		
		// 6. Return the file object
		return $verFile;
	}
	
	public function getVersion($uri, $version == null){
		$hisPath = $this->path . '/' . $this->historyFolder;
		
		// 1. Get the History 
		$hisVersion = $this->getHistoryByUri($uri);
		$hisLocation = $hisPath . '/' . $hisVersion);
		
		// 2. Get the version name
		$verNumber = '';
		if(is_null($version)){
			// Get the latest version
			$verNumber = $this->getNbVersion($hisLocation);
		} else {
			$verNumber = $version;
		}
		
		// 3. Build the version path
		$verPath = $hisLocation . '/' . $this->versionFolder . '/' . $verNumber;
		
		// 4. FileExist ??
		if(!file_exists($verPath)) throw new Sabre_DAV_Exception_FileNotFound('File could not be located');
		
		// 5. Create the file Object
		$verFile = new Sabre_DAV_FSExt_File($verPath);
		
		// 6. Return the file object
		return $verFile;
	}
	
	protected function getNbHistory(){
		
		$hisPath = $this->path . '/' . $this->historyFolder;
	
		$nbHistory = 0;
        foreach(scandir($hisPath) as $name){
			if($name!='.' && $name!='..' && $name!='.sabredav'){
				$path = $hisPath . '/' . $name;
				if (is_dir($path)) {
					$nbHistory ++;
				}
			}
		}
        return $nbHistory;
	}
	
	protected function getNbVersion($historyFolder){
		
		$hisPath = $this->path . '/' . $historyFolder;
	
		$nbVersion = 0;
        foreach(scandir($hisPath) as $name){
			if($name!='.' && $name!='..' && $name!='.sabredav'){
				$path = $hisPath . '/' . $name;
				if (!is_dir($path)) {
					$nbVersion ++;
				}
			}
		}
		
		if($nbVersion == 0) {
			// TODO: throw an exception, because the history can be empty
			return false;
		}
		
        return $nbVersion;
	}
	
	// The versioning behaviour are managed by the DeltaV database and not the Webdav, in case of Properties are not supported
	public function isUnderVersioning($uri){
	
		$uriVersion = $this->getData();
		
		// Return true if the $uri is a "version-controlled resource"
		// Return false if the $uri is a versionable resource
		if(array_key_exists($uri, $uriVersion)){
			return true;
		} else {
			return false;
		}

		// TODO: Throw an exception if the $uri can not be a "versionable resource" because is blacklisted [Not in the spec]
		// TODO: Throw an exception if the $uri are a "version resource"
	}
	
	
	public function checkin($uri){
	
	}
	
	public function checkout($uri){
	
	}
	
	private function registerUriVersion($uri, $his){
		$uriVersion = $this->getData();
		array_push($uriVersion, $uri=>$his);
		putData($uriVersion);
	}
	
	private function getHistoryByUri($uri){
		$uriVersion = $this->getData();
		if(array_key_exists($uri, $uriVersion)){
			return $uriVersion[$uri];
		} else {
			return false;
		}
	}
	
	
	
	
	
	/**
     * Returns the stored data for a uri
     *
     * @param string $uri
     * @return array 
     */
    protected function getData() {
		
		if(is_array($this->dataCache)) return $this->dataCache;
		
		$path = $this->dataDir . '/sabredeltav.dat';
		
        if (!file_exists($path)) return array();

        // opening up the file, and creating a shared lock
        $handle = fopen($path,'r');
        flock($handle,LOCK_SH);
        $data = '';

        // Reading data until the eof
        while(!feof($handle)) {
            $data.=fread($handle,8192);
        }

        // We're all good
        fclose($handle);

        // Unserializing and checking if the resource file contains data for this file
        $data = unserialize($data);
        if (!$data) return array();
		$this->dataCache = $data;
        return $data;

    }

    /**
     * Updates the lock information
     *
     * @param string $uri
     * @param array $newData 
     * @return void
     */
    protected function putData(array $newData) {
		
		$path = $this->dataDir . '/sabredeltav.dat';

        // opening up the file, and creating a shared lock
        $handle = fopen($path,'a+');
        flock($handle,LOCK_EX);
        ftruncate($handle,0);
        rewind($handle);

        fwrite($handle,serialize($newData));
        fclose($handle);
		
		$this->dataCache = $newData;

    }
	
}

?>